# -*- coding: utf-8 -*-

# pylint: disable=unused-variable,unused-argument,expression-not-assigned,singleton-comparison

from __future__ import unicode_literals

import pytest
from expecter import expect

from verchew.script import _, find_config, get_version, match_version, parse_config


def write(config, text, indent=8):
    config.write(text.replace(' ' * indent, ''))


def describe_find_config():

    @pytest.fixture
    def config(tmpdir):
        tmpdir.chdir()
        path = tmpdir.join("foo.bar")
        path.write("")
        return path

    def when_missing(config):
        with expect.raises(RuntimeError):
            find_config()

    def when_missing_and_generate(config):
        path = find_config(generate=True)

        generated_path = str(config).replace("foo.bar", "verchew.ini")
        expect(path) == generated_path

    def when_found(config):
        path = find_config(filenames=["foo.bar"])

        expect(path) == config


def describe_parse_config():

    @pytest.fixture
    def config(tmpdir):
        tmpdir.chdir()
        path = tmpdir.join("verchew.ini")
        path.write("")
        return path

    def with_no_content(config):
        config.write("")

        expect(parse_config(str(config))) == {}

    def with_an_empty_section(config):
        write(config, """
        [Foobar]
        """)

        expect(parse_config(str(config))) == {
            'Foobar': {
                'versions': '',
                'patterns': [''],
            },
        }

    def with_a_filled_section(config):
        write(config, """
        [Foobar]

        cli = foobar
        version = v1.2.3
        """)

        expect(parse_config(str(config))) == {
            'Foobar': {
                'cli': 'foobar',
                'versions': 'v1.2.3',
                'patterns': ['v1.2.3'],
            },
        }

    def with_multiple_versions(config):
        write(config, """
        [Foobar]

        version = 1
        versions = 2 | 3 |     4
        """)

        expect(parse_config(str(config))) == {
            'Foobar': {
                'versions': '2 | 3 |     4',
                'patterns': ['2', '3', '4'],
            },
        }


def describe_get_version():

    def when_missing():
        expect(get_version('foobar')) == "sh: command not found: foobar"

    def when_found():
        expect(get_version('python')).contains("Python ")

    def with_custom_argument():
        expect(get_version('python', argument='-V')).contains("Python ")

    def with_no_argument():
        expect(get_version('pip', argument='')).contains("Usage:")

    def when_no_output():
        expect(get_version('echo', argument='')) == ""


def describe_match_version():

    def when_exact_match():
        expect(match_version("1.2.3", "1.2.3")) == True

    def when_partial_match():
        expect(match_version("1.2.", "1.2.3")) == True

    def when_partial_match_at_boundary():
        expect(match_version("1.2", "1.2.3")) == True

    def when_mismatch():
        expect(match_version("1.", "2.0")) == False

    def when_mismatch_at_boundary():
        expect(match_version("1.2", "1.23")) == False

    def when_match_with_name():
        expect(match_version("1.", "Foobar 1.2.3")) == True

    def when_mismatch_with_name():
        expect(match_version("2.", "Foobar 1.2.3")) == False

    def when_match_with_version():
        expect(match_version("1.", "v1.2.3")) == True

    def when_mismatch_with_version():
        expect(match_version("2.", "v1.2.3")) == False


def describe_format():

    def default():
        expect(_('~')) == "~"

    @pytest.mark.parametrize("is_tty,supports_utf8,supports_ansi,formatted", [
        (0, 0, 0, "~"),
        (0, 0, 1, "~"),
        (0, 1, 0, "✔"),
        (0, 1, 1, "✔"),
        (1, 0, 0, "~"),
        (1, 0, 1, "\033[92m~\033[0m"),
        (1, 1, 0, "✔"),
        (1, 1, 1, "\033[92m✔\033[0m"),
    ])
    def with_options(is_tty, supports_utf8, supports_ansi, formatted):
        options = dict(is_tty=is_tty,
                       supports_utf8=supports_utf8,
                       supports_ansi=supports_ansi)

        expect(_('~', **options)) == formatted



"""Unit tests configuration file."""

import logging


def pytest_configure(config):
    """Disable verbose output when running tests."""
    logging.basicConfig(level=logging.DEBUG)

    terminal = config.pluginmanager.getplugin('terminal')
    base = terminal.TerminalReporter

    class QuietReporter(base):
        """A py.test reporting that only shows dots when running tests."""

        def __init__(self, *args, **kwargs):
            base.__init__(self, *args, **kwargs)
            self.verbosity = 0
            self.showlongtestinfo = False
            self.showfspath = False

    terminal.TerminalReporter = QuietReporter



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['verchew', 'verchew.tests']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['verchew = verchew.script:main']}

setup_kwargs = {
    'name': 'verchew',
    'version': '1.6.1',
    'description': 'System dependency version checker.',
    'long_description': "# Overview\n\n> ...chews through your system dependencies, spitting out incompatible versions.\n\nWhen onboarding new team members, ensuring their computer has everything needed to work on the project can be painful. Verchew is a command-line program and embeddable Python script to check the versions of your project's system dependencies. Its only external dependency is any Python interpreter, which should already be installed on macOS and most Linux-based operating systems.\n\n[![Unix Build Status](https://img.shields.io/travis/jacebrowning/verchew/develop.svg?label=unix)](https://travis-ci.org/jacebrowning/verchew)\n[![Windows Build Status](https://img.shields.io/appveyor/ci/jacebrowning/verchew/develop.svg?label=windows)](https://ci.appveyor.com/project/jacebrowning/verchew)\n[![Coverage Status](https://img.shields.io/coveralls/jacebrowning/verchew/develop.svg)](https://coveralls.io/r/jacebrowning/verchew)\n[![Scrutinizer Code Quality](https://img.shields.io/scrutinizer/g/jacebrowning/verchew.svg)](https://scrutinizer-ci.com/g/jacebrowning/verchew/?branch=develop)\n[![PyPI Version](https://img.shields.io/pypi/v/verchew.svg)](https://pypi.python.org/pypi/verchew)\n[![PyPI License](https://img.shields.io/pypi/l/verchew.svg)](https://pypi.org/project/verchew)\n\n# Setup\n\n## Requirements\n\n* Python 2.7+ or Python 3.3+\n\n## Installation\n\nInstall verchew with pip:\n\n```text\n$ pip install verchew\n```\n\nor directly from the source code:\n\n```text\n$ git clone https://github.com/jacebrowning/verchew.git\n$ cd verchew\n$ python setup.py install\n```\n\nor embedded the script in your project using [this guide](https://verchew.readthedocs.io/en/latest/cli/vendoring/).\n\n# Usage\n\nRun `verchew --init` to generate a sample configuration file.\n\nUpdate this file (`verchew.ini`) to include your project's system dependencies:\n\n```ini\n[Working Program]\n\ncli = working-program\nversion = 1.2\n\n[Newer Working Program]\n\ncli = working-program\nversions =  4.1 | 4.2\nmessage = Version 4.x is required to get the special features.\n\n[Broken Program]\n\ncli = broken-program\nversion = 1.2.3\n\n[Optional Missing Program]\n\ncli = missing-program\nversion = 1.2.3\noptional = true\n\n[Missing Program]\n\ncli = missing-program\nversion = 1.2.3\n```\n\nRun `verchew` to see if you have the expected versions installed:\n\n```text\n$ verchew\n\nChecking for Working Program...\n\n$ working-program --version\n1.2.3\n\xe2\x9c\x94 MATCHED: 1.2\n\nChecking for Newer Working Program...\n\n$ working-program --version\n1.2.3\n\xe2\x9c\x98 EXPECTED: 4.1 | 4.2\n\xe2\xad\x91 MESSAGE: Version 4.x is required to get the special features.\n\nChecking for Broken Program...\n\n$ broken-program --version\nAn error occurred.\n\xe2\x9c\x98 EXPECTED: 1.2.3\n\nChecking for Optional Missing Program...\n\n$ missing-program --version\nsh: command not found: missing-program\n\xe2\x9a\xa0 EXPECTED: 1.2.3\n\nChecking for Missing Program...\n\n$ missing-program --version\nsh: command not found: missing-program\n\xe2\x9c\x98 EXPECTED: 1.2.3\n\nResults: \xe2\x9c\x94 \xe2\x9c\x98 \xe2\x9c\x98 \xe2\x9a\xa0 \xe2\x9c\x98\n```\n",
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'url': 'https://pypi.org/project/verchew',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
}


setup(**setup_kwargs)
